#include <stdio.h>

#include "rjte.h"

/* How many timer1 overflows to require that a button be up
 * before it is considered to have been released.
 */
#define DEBOUNCE_COUNT		5

volatile u8 button_states;

/* This writes a character to the UART.
 * This is used for the stdio part of avr-libc.
 */
int uart_put(char ch)
{
	if (ch == '\n')
		uart_put('\r');

	loop_until_bit_is_set(UCSRA, UDRE);
	UDR = ch;

	return 0;
}

void initialize()
{
	/* Set up I/O ports */
	PORTB = 0xf0;
	PORTC = 0xff;
	PORTD = 0xfd;

	DDRB = 0x0f;
	DDRC = 0x00;
	DDRD = 0xe2;

	/* Set up the UART */
	UBRRH = 0;				/* Set 115200 baud for a 14.7456MHz crystal, U2X=0 */
	UBRRL = 7;
	UCSRA = (1 << TXC);			/* Clear TXC, turn off things we don't use */
	UCSRB = (1 << TXEN) | (1 << RXEN);	/* Turn on TX and RX */
	UCSRC = (1 << URSEL) | (3 << UCSZ0);	/* Asynchronous mode, 8-bit characters, no parity, 1 stop bit */

	/* Stop the motors */
	setMotorA(0, FORWARD);
	setMotorB(0, FORWARD);

	/* Set up timer1 for PWM (it's also used for button debouncing) */
	TCCR1A = 0xa1;
	TCCR1B = 0x03;

	/* Enable the timer1 overflow interrupt */
	TIMSK = (1 << TOIE1);

	/* Global interrupt enable */
	sei();

	/* stdio initialization */
	stderr = fdevopen(uart_put, NULL, 0);
}

void setMotorA(u8 speed, u8 direction)
{
	OCR1AH = 0;
	OCR1AL = speed;

	MOTOR_DIR_A = direction;
}

void setMotorB(u8 speed, u8 direction)
{
	OCR1BH = 0;
	OCR1BL = speed;

	MOTOR_DIR_B = direction;
}

/* This is called when timer1 (the motor PWM timer) overflows.
 * It's used to debounce the buttons.  Only button releases are debounced.
 */
SIGNAL(SIG_OVERFLOW1)
{
	static u8 count1, count2;

	/* Button 1 */
	if (BUTTON1_RAW)
	{
		/* Button is pressed: indicate immediately */
		button_states |= 1;
		count1 = DEBOUNCE_COUNT;
	} else {
		/* Button is released: Wait for several contiguous samples before changing the flag */
		if (count1 == 0)
		{
			button_states &= ~1;
		} else {
			count1--;
		}
	}

	/* Button 2 */
	if (BUTTON2_RAW)
	{
		/* Button is pressed: indicate immediately */
		button_states |= 2;
		count2 = DEBOUNCE_COUNT;
	} else {
		/* Button is released: Wait for several contiguous samples before changing the flag */
		if (count2 == 0)
		{
			button_states &= ~2;
		} else {
			count2--;
		}
	}
}

